@@ -24,13 +24,14 @@ from pymongo.objectid import ObjectId
 from pymongo.dbref import DBRef
 from pymongo.binary import Binary
 from errors import CorruptGridFile
+from pymongo import ASCENDING
 
 class GridFile(object):
     """A "file" stored in GridFS.
     """
     # TODO should be able to create a GridFile given a Collection object instead
     # of a database and collection name?
-    def __init__(self, file_spec, database, mode="r", collection="_files"):
+    def __init__(self, file_spec, database, mode="r", collection="fs"):
         """Open a "file" in GridFS.
 
         Application developers should generally not need to instantiate this
@@ -57,6 +58,7 @@ class GridFile(object):
           - "uploadDate": date when the object was first stored
             * only used for querying, automatically set for inserts
           - "aliases": array of alias strings
+          - "metadata": a SON document containing arbitrary data
 
         :Parameters:
           - `file_spec`: query specifier as described above
@@ -78,8 +80,9 @@ class GridFile(object):
             raise ValueError("mode must be one of ('r', 'w')")
 
         self.__collection = database[collection]
+        self.__collection.chunks.create_index([("files_id", ASCENDING), ("n", ASCENDING)])
 
-        file = self.__collection.find_one(file_spec)
+        file = self.__collection.files.find_one(file_spec)
         if file:
             self.__id = file["_id"]
         else:
@@ -88,42 +91,25 @@ class GridFile(object):
             file_spec["length"] = 0
             file_spec["uploadDate"] = datetime.datetime.now()
             file_spec.setdefault("chunkSize", 256000)
-            self.__id = self.__collection.insert(file_spec)["_id"]
+            self.__id = self.__collection.files.insert(file_spec)["_id"]
 
         self.__mode = mode
         if mode == "w":
             self.__erase()
-            self.__current_chunk = None
-        elif mode == "r":
-            self.__position = 0
-            self.__current_chunk = file["next"]
-            self.__read_buffer = ""
+        self.__buffer = ""
+        self.__position = 0
+        self.__chunk_number = 0
         self.__closed = False
 
     def __erase(self):
         """Erase all of the data stored in this GridFile.
         """
-        file = self.__collection.find_one(self.__id)
-
-        next = file.get("next", None)
-        chunk_number = 0
-
-        while next:
-            chunk = self.__collection.database().dereference(next)
-            if not chunk:
-                raise CorruptGridFile("could not dereference: %r" % next)
-            if chunk["cn"] != chunk_number:
-                raise CorruptGridFile("incorrect chunk number: %r, should be: %r" %
-                                      (chunk["cn"], chunk_number))
-
-            self.__collection.database()[next.collection()].remove({"_id": next.id()})
-
-            chunk_number += 1
-            next = chunk["next"]
-
+        file = self.__collection.files.find_one(self.__id)
         file["next"] = None
         file["length"] = 0
-        self.__collection.save(file)
+        self.__collection.files.save(file)
+
+        self.__collection.chunks.remove({"files_id": self.__id})
 
     @property
     def closed(self):
@@ -135,11 +121,11 @@ class GridFile(object):
 
     def __create_property(field_name, read_only=False):
         def get(self):
-            return self.__collection.find_one(self.__id).get(field_name, None)
+            return self.__collection.files.find_one(self.__id).get(field_name, None)
         def set(self, value):
-            file = self.__collection.find_one(self.__id)
+            file = self.__collection.files.find_one(self.__id)
             file[field_name] = value
-            self.__collection.save(file)
+            self.__collection.files.save(file)
         if not read_only:
             return property(get, set)
         return property(get)
@@ -150,7 +136,7 @@ class GridFile(object):
     chunk_size = __create_property("chunkSize", True)
     upload_date = __create_property("uploadDate", True)
     aliases = __create_property("aliases")
-    next = __create_property("next", True)
+    metadata = __create_property("metadata")
 
     def rename(self, filename):
         """Rename this GridFile.
@@ -161,30 +147,51 @@ class GridFile(object):
         :Parameters:
           - `filename`: the new name for this GridFile
         """
-        file = self.__collection.find_one(self.__id)
+        file = self.__collection.files.find_one(self.__id)
         file["filename"] = filename
-        self.__collection.save(file)
+        self.__collection.files.save(file)
 
-    __chunks_collection = "_chunks"
-    def __write_current_chunk(self):
-        # TODO _chunks collection should be configurable?
-        self.__current_chunk["data"] = Binary(self.__current_chunk["data"])
-        self.__collection.database()[self.__chunks_collection].save(self.__current_chunk)
+    def __max_chunk(self):
+        return self.__collection.chunks.find_one({"files_id": self.__id, "n": self.__chunk_number})
+
+    def __new_chunk(self, n):
+        chunk = {"files_id": self.__id,
+                 "n": n,
+                 "data": ""}
+        self.__collection.chunks.insert(chunk)
+        return chunk
+
+    def __write_buffer_to_chunks(self):
+        """Write the buffer contents out to chunks.
+        """
+        while len(self.__buffer):
+            max = self.__max_chunk()
+            if not max:
+                max = self.__new_chunk(self.__chunk_number)
+            space = (self.__chunk_number + 1) * self.chunk_size - self.__position
+            if not space:
+                self.__chunk_number += 1
+                max = self.__new_chunk(self.__chunk_number)
+                space = self.chunk_size
+            to_write = len(self.__buffer) > space and space or len(self.__buffer)
+
+            max["data"] = Binary(max["data"] + self.__buffer[:to_write])
+            self.__collection.chunks.save(max)
+            self.__buffer = self.__buffer[to_write:]
+            self.__position += to_write
 
     def flush(self):
         """Flush the GridFile to the database.
         """
         self.__assert_open()
-        if self.mode != "w" or not self.__current_chunk:
+        if self.mode != "w":
             return
 
-        file = self.__collection.find_one(self.__id)
+        file = self.__collection.files.find_one(self.__id)
+        file["length"] = self.__position + len(self.__buffer)
+        self.__collection.files.save(file)
 
-        length = file["chunkSize"] * self.__current_chunk["cn"] + len(self.__current_chunk["data"])
-        file["length"] = length
-
-        self.__write_current_chunk()
-        self.__collection.save(file)
+        self.__write_buffer_to_chunks()
 
     def close(self):
         """Close the GridFile.
@@ -217,30 +224,23 @@ class GridFile(object):
         if size == 0:
             return ""
 
-        remainder = self.length - self.__position
+        remainder = int(self.length) - self.__position
         if size < 0 or size > remainder:
             size = remainder
 
-        bytes = self.__read_buffer
+        bytes = self.__buffer
         chunk_number = math.floor(self.__position / self.chunk_size)
+
         while len(bytes) < size:
-            if not self.__current_chunk:
-                raise CorruptGridFile("incorrect length for file: %r" % self)
-            chunk = self.__collection.database().dereference(self.__current_chunk)
+            chunk = self.__collection.chunks.find_one({"files_id": self.__id, "n": chunk_number})
             if not chunk:
-                raise CorruptGridFile("could not dereference: %r" % self.__current_chunk)
-            if chunk["cn"] != chunk_number:
-                raise CorruptGridFile("incorrect chunk number: %r, should be: %r" %
-                                      (chunk["cn"], chunk_number))
-
+                raise CorruptGridFile("no chunk for n = " + chunk_number)
             bytes += chunk["data"]
-
             chunk_number += 1
-            self.__current_chunk = chunk["next"]
 
         self.__position += size
         to_return = bytes[:size]
-        self.__read_buffer = bytes[size:]
+        self.__buffer = bytes[size:]
         return to_return
 
     # TODO should support writing unicode to a file. this means that files will
@@ -264,35 +264,7 @@ class GridFile(object):
         if not len(str):
             return
 
-        def initialize_chunk(number):
-            id = ObjectId()
-            new_chunk = SON([("_id", id),
-                             ("cn", number),
-                             ("data", ""),
-                             ("next", None)])
-            return new_chunk
-
-        if not self.__current_chunk:
-            self.__current_chunk = initialize_chunk(0)
-            ref = DBRef(self.__chunks_collection, self.__current_chunk["_id"])
-            file = self.__collection.find_one(self.__id)
-            file["next"] = ref
-            self.__collection.save(file)
-
-        data = self.__current_chunk["data"]
-        data += str
-
-        while len(data) > self.chunk_size:
-            self.__current_chunk["data"] = data[:self.chunk_size]
-            data = data[self.chunk_size:]
-
-            new_chunk = initialize_chunk(self.__current_chunk["cn"] + 1)
-
-            self.__current_chunk["next"] = DBRef(self.__chunks_collection, new_chunk["_id"])
-            self.__write_current_chunk()
-            self.__current_chunk = new_chunk
-
-        self.__current_chunk["data"] = data
+        self.__buffer += str
 
     def writelines(self, sequence):
         """Write a sequence of strings to the file.
