@@ -31,6 +31,12 @@ from dbref import DBRef
 from son import SON
 from errors import InvalidBSON, InvalidDocument, UnsupportedTag
 
+try:
+    import _cbson
+    _use_c = True
+except ImportError:
+    _use_c = False
+
 _logger = logging.getLogger("pymongo.bson")
 # _logger.setLevel(logging.DEBUG)
 # _logger.addHandler(logging.StreamHandler())
@@ -198,14 +204,14 @@ def _get_array(data):
 
 def _get_binary(data):
     (length, data) = _get_int(data)
-    subtype = data[0]
+    subtype = ord(data[0])
     data = data[1:]
-    if subtype != "\x02":
-        raise InvalidBSON("binary subtype %r is unsupported" % subtype)
-    (length2, data) = _get_int(data)
-    if length2 != length - 4:
-        raise InvalidBSON("invalid binary - lengths don't match!")
-    return (Binary(data[:length2]), data[length2:])
+    if subtype == 2:
+        (length2, data) = _get_int(data)
+        if length2 != length - 4:
+            raise InvalidBSON("invalid binary (st 2) - lengths don't match!")
+        length = length2
+    return (Binary(data[:length], subtype), data[length:])
 
 def _get_oid(data):
     oid = _shuffle_oid(data[:12])
@@ -283,15 +289,18 @@ def _document_to_dict(data):
 
 def _shuffle_oid(data):
     return data[7::-1] + data[:7:-1]
+if _use_c:
+    _shuffle_oid = _cbson._shuffle_oid
 
 _RE_TYPE = type(_valid_array_name)
-def _element_to_bson(key, value):
-    name = _make_c_string(key)
+def _element_to_bson(name, value):
     if isinstance(value, float):
         return "\x01" + name + struct.pack("<d", value)
     if isinstance(value, Binary):
-        length = len(value)
-        return "\x05" + name + struct.pack("<i", length + 4) + "\x02" + struct.pack("<i", length) + value
+        subtype = value.subtype()
+        if subtype == 2:
+            value = struct.pack("<i", len(value)) + value
+        return "\x05" + name + struct.pack("<i", len(value)) + chr(subtype) + value
     if isinstance(value, Code):
         cstring = _make_c_string(value, False)
         length = struct.pack("<i", len(cstring))
@@ -342,6 +351,8 @@ def _element_to_bson(key, value):
         ns = _make_c_string(value.collection())
         return "\x0C" + name + struct.pack("<i", len(ns)) + ns + _shuffle_oid(str(value.id()))
     raise InvalidDocument("cannot convert value of type %s to bson" % type(value))
+# if _use_c:
+#     _element_to_bson = _cbson._element_to_bson
 
 def is_valid(bson):
     """Validate that the given string represents valid BSON data.
@@ -402,7 +413,7 @@ class BSON(str):
           - `dict`: mapping type representing a Mongo document
         """
         try:
-            elements = "".join([_element_to_bson(key, value) for (key, value) in dict.iteritems()])
+            elements = "".join([_element_to_bson(_make_c_string(key), value) for (key, value) in dict.iteritems()])
         except AttributeError:
             raise TypeError("argument to from_dict must be a mapping type")
 
