@@ -117,35 +117,43 @@ class Collection(object):
         """
         return self.__database
 
-    def save(self, to_save, manipulate=True):
+    def save(self, to_save, manipulate=True, safe=False):
         """Save a SON object in this collection.
 
-        Raises TypeError if to_save is not an instance of (dict, SON).
+        Raises TypeError if to_save is not an instance of (dict, SON). If `safe`
+        is True then the save will be checked for errors, raising
+        OperationFailure if one occurred. Checking for safety requires an extra
+        round-trip to the database.
 
         :Parameters:
           - `to_save`: the SON object to be saved
           - `manipulate` (optional): manipulate the son object before saving it
+          - `safe` (optional): check that the save succeeded?
         """
         if not isinstance(to_save, (types.DictType, SON)):
             raise TypeError("cannot save object of type %s" % type(to_save))
 
         if "_id" not in to_save:
-            result = self.insert(to_save, manipulate)
+            result = self.insert(to_save, manipulate, safe)
             return result.get("_id", None)
         else:
-            self.update({"_id": to_save["_id"]}, to_save, True, manipulate)
+            self.update({"_id": to_save["_id"]}, to_save, True, manipulate, safe)
             return to_save.get("_id", None)
 
-    def insert(self, doc_or_docs, manipulate=True):
+    def insert(self, doc_or_docs, manipulate=True, safe=False):
         """Insert a document(s) into this collection.
 
         If manipulate is set the document(s) are manipulated using any
         SONManipulators that have been added to this database. Returns the
-        inserted object or a list of inserted objects.
+        inserted object or a list of inserted objects. If `safe`is True then the
+        insert will be checked for errors, raising OperationFailure if one
+        occurred. Checking for safety requires an extra round-trip to the
+        database.
 
         :Parameters:
           - `doc_or_docs`: a SON object or list of SON objects to be inserted
           - `manipulate` (optional): monipulate the objects before inserting?
+          - `safe` (optional): check that the insert succeeded?
         """
         docs = doc_or_docs
         if isinstance(docs, (types.DictType, SON)):
@@ -160,13 +168,21 @@ class Collection(object):
         data = [bson.BSON.from_dict(doc) for doc in docs]
         self._send_message(2002, "".join(data))
 
+        if safe:
+            error = self.__database.error()
+            if error:
+                raise OperationFailure("insert failed: " + error["err"])
+
         return len(docs) == 1 and docs[0] or docs
 
-    def update(self, spec, document, upsert=False, manipulate=True):
+    def update(self, spec, document, upsert=False, manipulate=True, safe=False):
         """Update an object(s) in this collection.
 
         Raises TypeError if either spec or document isn't an instance of
-        (dict, SON) or upsert isn't an instance of bool.
+        (dict, SON) or upsert isn't an instance of bool. If `safe` is True then
+        the update will be checked for errors, raising OperationFailure if one
+        occurred. Checking for safety requires an extra round-trip to the
+        database.
 
         :Parameters:
           - `spec`: a SON object specifying elements which must be present for a
@@ -176,6 +192,7 @@ class Collection(object):
             be inserted.
           - `upsert` (optional): perform an upsert operation
           - `manipulate` (optional): monipulate the document before updating?
+          - `safe` (optional): check that the update succeeded?
         """
         if not isinstance(spec, (types.DictType, SON)):
             raise TypeError("spec must be an instance of (dict, SON)")
@@ -184,7 +201,7 @@ class Collection(object):
         if not isinstance(upsert, types.BooleanType):
             raise TypeError("upsert must be an instance of bool")
 
-        if manipulate:
+        if upsert and manipulate:
             document = self.__database._fix_incoming(document, self)
 
         message = upsert and _ONE or _ZERO
@@ -193,6 +210,11 @@ class Collection(object):
 
         self._send_message(2001, message)
 
+        if safe:
+            error = self.__database.error()
+            if error:
+                raise OperationFailure("update failed: " + error["err"])
+
     def remove(self, spec_or_object_id):
         """Remove an object(s) from this collection.
 
