@@ -15,7 +15,8 @@
 """Tools for creating and manipulating SON, the Serialized Ocument Notation.
 
 Regular dictionaries can be used instead of SON objects, but not when the order
-of keys is important."""
+of keys is important. A SON object can be used just like a normal Python
+dictionary."""
 
 import datetime
 import re
@@ -25,6 +26,7 @@ from UserDict import DictMixin
 
 import elementtree.ElementTree as ET
 
+from code import Code
 from binary import Binary
 from objectid import ObjectId
 from dbref import DBRef
@@ -57,6 +59,7 @@ class SON(DictMixin):
     `pymongo.dbref.DBRef`        dbref          both
     None                         undefined      mongo -> py
     unicode                      code           mongo -> py
+    `pymongo.code.Code`          code           py -> mongo
     unicode                      symbol         mongo -> py
     ===========================  =============  ===================
 
@@ -143,6 +146,9 @@ class SON(DictMixin):
         def make_string(string):
             return string.text is not None and unicode(string.text) or u""
 
+        def make_code(code):
+            return code.text is not None and Code(code.text) or Code("")
+
         def make_binary(binary):
             return binary.text is not None and Binary(base64.b64decode(binary.text)) or Binary("")
 
@@ -195,7 +201,7 @@ class SON(DictMixin):
                         "string": make_string,
                         "binary": make_binary,
                         "boolean": make_boolean,
-                        "code": make_string,
+                        "code": make_code,
                         "date": make_date,
                         "ref": make_ref,
                         "ns": make_string,
