@@ -16,6 +16,8 @@
 
 import unittest
 import types
+import sys
+sys.path[0:0] = [""]
 
 from pymongo.objectid import ObjectId
 from pymongo.dbref import DBRef
